/*******************************
 *          __                   
 *         /\ \                  
 *  __  _  \_\ \     __    ___   
 * /\ \/'\ /'_' \  /'__'\ /'___\ 
 * \/>  <//\ \ \ \/\  __//\ \__/ 
 *  /\_/\_\ \___,_\ \____\ \____\
 *  \//\/_/\/__,_ /\/____/\/____/
 * 
 *  Author: declis (xdec.de)
 ********************************/ 

#include <msp430g2452.h>
#include "lib_dcf77.h"
#include "lib_lcd.h"
#include "delay.h"

void main(void)
{
	unsigned int count=0,count_low=0;
	unsigned char data[8];
	unsigned char bit_num=0,x=0,y=4;
	unsigned char start=0,no_signal=0;

	WDTCTL=WDTPW+WDTHOLD;					// stop WDT
	BCSCTL1=CALBC1_1MHZ;       				// SMCLK ~1MHz
  	DCOCTL=CALDCO_1MHZ;
	P1DIR&=~dcf_d_port;						// DCF77 data port
	
  	init_USI();
  	init_LCD();
  	
  	write_char(0,2,antenna_sym,1);			// Antenna symbol
  	
	while(1)
	{
		write_string(x,y,wait_sync,0);
		// wait for start condition
		// (59th bit is always low for 1000ms) 
		while(!start)	
		{		
			// while data port in low state -> count time
			// if "no_signal" is set, we will wait for a changing level at data port 
			while(!(P1IN&dcf_d_port)&&!no_signal)	
			{
				wait_ms(1);					// wait ~1ms
				count++;					// count time (x*1ms)
				
				if(count>2000)				// if no changing level detected -> no signal
				{							
					count=0;				// reset count (low time)
					no_signal=1;			// set "no_signal"
					write_char(sig_sym_x,sig_sym_y,'x',1);	// "no signal" symbol	
				}
			}
			// if "no_signal" is set, wait for changing level at data port 
			// this will not detect a bad (malfunction) signal
			if(no_signal&&(P1IN&dcf_d_port))
			{
				no_signal=0;				// signal detected, reset no_signal
				count=0;					// reset count (low time)
			}
			// low time must be >1s (59th bit always zero for 1s, plus 0. Bit (800 or 900ms)
			if(count>1000&&count<2000&&!no_signal)
			{								 
				start=1;					// a new minute started
				count=0;		
			}
			// if time is not in range (maybe no sync bit detected or no signal)
			else 
			{
				count=0;					// wait and count again
				if(!no_signal) 				// signal seems to be OK	
					write_char(sig_sym_x,sig_sym_y,sig_sym,1); 	
				else write_char(sig_sym_x,sig_sym_y,'x',1);	// no signal	
			}
		}

		clean_area(x,6*(sizeof(wait_sync)-1),y,y);
		
		// stop at bit 58
		while(bit_num<=58)
		{
			// count time if data port is high
			while(P1IN&dcf_d_port)			
			{
				wait_ms(1);
				count++;	
			}
			// if counted time ~100ms -> logical 0
			if(count>80&&count<120)			
			{
				data[bit_num/8]&=~(1<<bit_num%8);	
				char2num_out(14,7,bit_num,0);
				bit_num++;		
				count_low=0;
				write_char(x++,y,'0',0);	
			}
			// if counted time ~200ms -> logical 1
			if(count>180&&count<220)		
			{
				data[bit_num/8]|=(1<<bit_num%8);
				char2num_out(14,7,bit_num,0);
				bit_num++;
				count_low=0;
				write_char(x++,y,'1',0);
			}			
			// check low time of data port
			if(!(P1IN&dcf_d_port))
			{
				wait_ms(1);
				count_low++;
				// signal lost, start at sync loop again
				if(count_low>1000)
				{
					count_low=0;
					write_char(sig_sym_x,sig_sym_y,'x',1);
					bit_num=59; // will break while()
					no_signal=1;
				}
			}
			// display width check
			if(x>15)
			{
				 y++; 
				 x=0;
			}	
			count=0;
		}
	
		if(!no_signal)
		{
			if(parity_test(29,35,data)||
				parity_test(21,28,data)||
				parity_test(36,58,data))
				write_char(15,7,'x',0);		// parity test only for fun
			else
			{
				char2num_out(4,0,bit_decode(29,34,data),2);	// hour
				write_char(8,0,':',2);
				char2num_out(10,0,bit_decode(21,27,data),2);// minute
				char2num_out(3,2,bit_decode(36,41,data),1);	// day
				write_char(5,2,'.',1);
				char2num_out(6,2,bit_decode(45,49,data),1);	// month
				write_char(8,2,'.',1);
				char2num_out(9,2,bit_decode(50,57,data),1);	// year
				write_char(12,2,'(',1);
				write_string(13,2,day_name[bit_decode(42,44,data)-1],1);	
				write_char(15,2,')',1);
			}
		}
		count_low=0;
		count=0;
		start=0;
		bit_num=0;
		x=0;
		y=4;
		clean_area(0,lcd_width-1,y,lcd_height_b-1);
	}
}
